/*
 * labyrinth with wii nnchuck for Ping-Pong Board from FRANZIS / conrad.de
 *
 *
 * Ver.     Date         Author           Comments
 * -------  ----------   --------------   ------------------------------
 * 1.00	    07.02.2010   Piotr Platek     initial
 *
 *
 * Feel free to provide me any feedback 
 ***************************************************************************
 **
 **  This program is free software; you can redistribute it and/or
 **  modify it under the terms of the GNU General Public License
 **  as published by the Free Software Foundation; either version 2
 **  of the License, or (at your option) any later version.
 **
 **  This program is distributed in the hope that it will be useful,
 **  but WITHOUT ANY WARRANTY; without even the implied warranty of
 **  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **  GNU General Public License for more details.
 **
 **  You should have received a copy of the GNU General Public License
 **  along with this program; if not, write to the Free Software Foundation, 
 **  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ***************************************************************************
 */

#include "main.h"

uint8_t buffor[40] = "";	// buffor for text display & manipulation

void DrawBoard(uint8_t bx, uint8_t by)
{
  uint8_t x, xend, mody, divy, colx;
  uint16_t screen;

  xend = bx+SWIDTH;

//  if (by > SizeY*SHIGH-1 ) by = SizeY*SHIGH-1;
//  if (bx > SizeX-SWIDTH-1) bx = SizeX-SWIDTH-1;

  divy = by / SHIGH;
  mody = by % SHIGH;
  
  colx = 0;
  for (x=bx; x < xend; x++)
  {
	screen = pgm_read_word(&gPrgBoard[BoardNumber+divy][x]) << mody;
	if (mody)
	{
		screen |= (pgm_read_word(&gPrgBoard[BoardNumber+divy+1][x]) >> (SHIGH-mody));
	}
	gLEDs[colx++]=screen;
  }
}


int8_t GetPixelBoard(uint8_t bx, uint8_t by)
{
  uint8_t mody, divy;

  divy = by / SHIGH;
  mody = by % SHIGH;
  
  if (pgm_read_word(&gPrgBoard[BoardNumber+divy][bx]) & ( 1 << (SHIGH-1 - mody)))
  	return 1 ;
  else
  	return 0 ;
}


int main(void)
{


	cli();

#ifdef DEBUGON
	USART_Init();
#endif

	DDRC = 0xFF;   // port B, C, D as Output
	DDRB = 0xFF;   // TWI/C2I will be set by nunchuckInit/TWI funnctions
	DDRD = 0xFF;

	/* initialize timers */
	TCCR0 |= _BV(CS01) | _BV(CS00); // clk prescale (clk/64)
	TIMSK |= _BV(TOIE0); 			// enable timer/counter0 overflow interrupt 
	TCNT0 = 0;           			// reset the timer

	ADMUX = _BV(MUX2) | _BV(MUX1) | _BV(MUX0);								// Initially we start with the ADC07
	ADCSRA = _BV(ADEN) | _BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0);				// Enable ADC + division factor of 128, we do not need high speed
	ADCSRA |= _BV(ADSC);													// Start the first conversion

	sei();


  int8_t sx,sy, px, py, tx, ty, rx, ry, dx, dy, lx, ly, hx, hy, BoardIndex, found;
  uint16_t GameTimeDef;

  BoardIndex = BOARDN-1-1; //let's start from the second board, the first is to easy!


  gametime = 10;
  while (gametime)
  {
    UpdatePoti(); // read data from Potis
	sprintf((char*)(buffor), "%d", joyX);
	LedPrint(buffor, 0, 5);
	sprintf((char*)(buffor), "%d", joyY);
	LedPrint(buffor, 0, 0);
	_delay_ms(100);
	LedCLS();
  }

  LedCLS();
  	
  while (1)
  {
    found = 0;
	sx = 0;
    sy = 0;
    px = 1;
    py = 1;
    lx = 1;
    ly = 1;
    huntblink = HUNTBLINKS;
    playerblink = PLAYERBLINKS;
	  
    hx = (int8_t)(pgm_read_word(&gPrgHunt[BoardIndex][0]));
	hy = (int8_t)(pgm_read_word(&gPrgHunt[BoardIndex][1]));
	GameTimeDef = pgm_read_word(&gPrgHunt[BoardIndex][2]);

    LedCLS();
    BoardNumber = BoardIndex*SizeY;
    DrawBoard(sx,sy);
	gametime = GameTimeDef;
   

    while(gametime && !(found))
    {
		UpdatePoti(); // read data from Potis
//		sprintf(buffor, "%d,%d,%d,%d,%d,%d,%d\n\r",accelX, accelY, accelZ, joyX, joyY, butC, butZ);
//		sprintf((char *)(buffor), "%d,%d,%d,%d,%d,%d,%d\n\r",sensors[2], sensors[3], sensors[4], sensors[0], sensors[1], butC, butZ);
//		sprintf(buffor, "%d,%d,%d,%d\r\n", accelX, accelY, accelZ, sensors[2]);
//		sprintf((char *)(buffor), "%d,%d,%d\r\n", accelX, accelY, accelZ);
//		sprintf(buffor, "%d,%d,%d\r\n", sensors[2], sensors[3], sensors[4]);

//		sprintf(buffor, "%d,%d\n\r",joyX, joyY);
//		rsprint(buffor);


		dx = 0;
		dy = 0;

		if (joyX < -30) dx = +1;			// in case of small movement keep the same position
		if (joyX >  30) dx = -1;

		if (joyY < -30) dy = +1;
		if (joyY >  30) dy = -1;

		tx = px+dx;
		ty = py+dy;
		if ( ( ((tx >= 0) || (tx < SizeX)) && ( (ty>=0) || (ty<SizeY) ) ) && (dx || dy))
		{
			rx = lx;
			ry = ly;
			if (!GetPixelBoard(tx,py))
			{
				if ((lx+dx < GAP) && (dx < 0) && (px+dx >= GAP) )
				{
					sx += dx;
				}
				else if ((lx+dx >= SWIDTH-GAP) && (dx > 0) && (px+dx<SizeX-GAP))
				{
					sx += dx;
				}
				else 
				{
				  lx += dx;
				}
				px += dx;			 
             }

			if (!GetPixelBoard(px,ty))
			{
				if ((ly+dy < GAP) && (dy < 0) && (py+dy >= GAP) )
				{
					sy += dy;
				}
				else if ((ly+dy >= SHIGH-GAP) && (dy > 0) && (py+dy<SizeY*SHIGH-GAP))
				{
					sy += dy;
				}
				else 
				{
				  	ly += dy;
				}
				py += dy;
			}
			if (sy > (SizeY-1)*SHIGH) sy = (SizeY-1)*SHIGH;
  			if (sx > SizeX-SWIDTH) sx = SizeX-SWIDTH;
			if (sy < 0) sy = 0;
			if (sx < 0) sx = 0;
			DrawBoard(sx,sy);
			ClearPixel(rx, ry);
			SetPixel(lx,ly);
			if ( ( px == hx ) && ( py == hy ) )
			{
				found = 1;
            }
		}
		if (!(playerblink))			 // small counter to blink player position
		{
			XorPixel(lx, ly);		 // inverse curent ball status
			playerblink = PLAYERBLINKS;	 // restart timer
		}


		if ( (abs(hx-sx) <= SWIDTH-1) && (abs(hy-sy)<=SHIGH) && (hx-sx >= 0) && (hy-sy >= 0))
		{

			if (!(huntblink))			 // small counter to blink player position
			{
				XorPixel(abs(hx-sx), abs(hy-sy));		 // inverse curent ball status
				huntblink = HUNTBLINKS;	 // restart timer
			}
		}
/*		if (!butC) 			// if C pressed move to the initial position
		{
		  sx = 0;
  		  sy = 0;
  		  px = 1;
          py = 1;
  		  lx = 1;
          ly = 1;
	 	  gametime = GameTimeDef;
		  DrawBoard(sx,sy);
		}
*/
		_delay_ms(100);				// wait a bit 
      }
	  LedCLS();
	  if (found)
	  {
	  	sprintf((char*)(buffor), "OK!");
		  BoardIndex--;
		  if (BoardIndex <0) BoardIndex = BOARDN-1;
	  }
	  else 
	  	sprintf((char*)(buffor), " :(");
	  LedPrint(buffor, 0,2);
	  _delay_ms(5000);
	}
	return 0;
}
